@@ -16,8 +16,7 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_ALGORITHM_BOUNDARYNODERULE_H
-#define GEOS_ALGORITHM_BOUNDARYNODERULE_H
+#pragma once
 
 #include <geos/export.h>
 
@@ -141,5 +140,3 @@ public:
 } // namespace geos::algorithm
 } // namespace geos
 
-#endif // GEOS_ALGORITHM_BOUNDARYNODERULE_H
-
